@@ -59,7 +59,7 @@ int want_skill_exp = 0, replyinfo_status = 0, requestinfo_sent = 0,
 
 /* Use the 'new' login method by default */
 int wantloginmethod = 2;
-int serverloginmethod = 0;
+int serverloginmethod = -1;
 
 guint16 exp_table_max=0;
 guint64 *exp_table=NULL;
@@ -203,7 +203,7 @@ void client_run() {
     GError* err = NULL;
     SockList inbuf;
     inbuf.buf = g_malloc(MAXSOCKBUF);
-    if (!SockList_ReadPacket(csocket.fd, &inbuf, MAXSOCKBUF - 1, &err)) {
+    if (!SockList_ReadPacket(in, &inbuf, MAXSOCKBUF - 1, &err)) {
     /*
      * If a socket error occurred while reading the packet, drop the
      * server connection.  Is there a better way to handle this?
@@ -217,6 +217,7 @@ void client_run() {
     }
     if (inbuf.len == 0) {
         client_disconnect();
+        error_dialog("Disconnected", "The server closed the connection.");
         return;
     }
     /*
@@ -326,7 +327,8 @@ GSource *client_get_source() {
             G_POLLABLE_INPUT_STREAM(in), NULL);
 }
 
-void client_negotiate(int sound) {
+bool client_negotiate(int sound) {
+    serverloginmethod = -1; // how we detect if server replied to our setup command
     int tries;
 
     SendVersion(csocket);
@@ -341,16 +343,17 @@ void client_negotiate(int sound) {
     while (csocket.cs_version==0) {
         client_run();
         if (csocket.fd == NULL) {
-            return;
+            LOG(LOG_ERROR, "client_negotiate", "Server disconnected while waiting for version command");
+            return false;
         }
 
         usleep(10*1000);    /* 10 milliseconds */
         tries++;
         /* If we haven't got a response in 10 seconds, bail out */
         if (tries > 1000) {
-            LOG (LOG_ERROR,"common::negotiate_connection", "Connection timed out");
+            LOG(LOG_ERROR,"client_negotiate", "Timed out waiting for version command");
             client_disconnect();
-            return;
+            return false;
         }
     }
 
@@ -376,8 +379,9 @@ void client_negotiate(int sound) {
      * client prefers is last.
      */
     cs_print_string(csocket.fd, "setup "
-            "map2cmd 1 tick 1 sound2 %d darkness %d spellmon 1 spellmon 2 "
+            "map2cmd 1 tick %d sound2 %d darkness %d spellmon 1 spellmon 2 "
             "faceset %d facecache %d want_pickup 1 loginmethod %d newmapcmd 1 extendedTextInfos 1",
+            want_config[CONFIG_SERVER_TICKS],
             (sound >= 0) ? 3 : 0, want_config[CONFIG_LIGHTING] ? 1 : 0,
             face_info.faceset, want_config[CONFIG_CACHE], wantloginmethod);
 
@@ -419,7 +423,8 @@ void client_negotiate(int sound) {
              * It is rare, but the connection can die while getting this info.
              */
             if (csocket.fd == NULL) {
-                return;
+                LOG(LOG_ERROR,"client_negotiate", "Server disconnected while downloading faces");
+                return false;
             }
 
             if (use_config[CONFIG_DOWNLOAD]) {
@@ -476,6 +481,17 @@ void client_negotiate(int sound) {
         draw_ext_info(NDI_GOLD, MSG_TYPE_CLIENT, MSG_TYPE_CLIENT_CONFIG, buf);
     }
 
+    // Block until server replies to our setup command.
+    while (serverloginmethod == -1) {
+        client_run();
+        if (csocket.fd == NULL) {
+            LOG(LOG_ERROR,"client_negotiate", "Server disconnected while waiting for setup");
+            return false;
+        }
+
+        usleep(10*1000);
+    }
+
     /* This needs to get changed around - we really don't want to send the
      * SendAddMe until we do all of our negotiation, which may include things
      * like downloading all the images and whatnot - this is more an issue if
@@ -483,7 +499,9 @@ void client_negotiate(int sound) {
      * end up building images from the wrong set.
      * Only run this if not using new login method
      */
-    if (!serverloginmethod) {
+    if (serverloginmethod == 0) {
         SendAddMe(csocket);
     }
+
+    return true;
 }
