@@ -65,7 +65,7 @@ typedef struct TextManager{
 #define DEFPORT "13327"
 
 #define VERSION_CS 1023
-#define VERSION_SC 1029
+#define VERSION_SC 1030
 
 extern char VERSION_INFO[256];
 
@@ -214,7 +214,10 @@ typedef enum {
 #define CONFIG_TIMESTAMP        31
 #define CONFIG_AUTO_AFK         32
 #define CONFIG_INV_MENU         33
-#define CONFIG_NUMS             34      /**< This should always be the last
+#define CONFIG_MUSIC_VOL        34
+#define CONFIG_SERVER_TICKS     35      //< Send "setup tick 1" for the server to send ticks, else generate our own
+#define CONFIG_DEBOUNCE         36
+#define CONFIG_NUMS             37      /**< This should always be the last
                                              value in the CONFIG_xxx list. */
 /*@}*/
 
@@ -639,6 +642,12 @@ enum ClientError {
 /* We need to declare most of the structs before we can include this */
 #include "proto.h"
 
+/**
+ * Current number of layers to render for renderer debugging purposes.
+ * Normally set to MAXLAYERS when not debugging.
+ */
+extern int render_debug_layers;
+
 /**
  * Open a socket to the given hostname and store connection information.
  *
@@ -662,7 +671,7 @@ extern void client_disconnect(void);
  *
  * @param sound Non-zero to ask for sound and music commands.
  */
-extern void client_negotiate(int sound);
+extern bool client_negotiate(int sound);
 
 /**
  * Ask the server for the given map size.
