@@ -1062,6 +1062,7 @@ void SetupCmd(char *buf, int len)
              * version the server supports so client can do appropriate
              * work
              */
+            serverloginmethod = method;
             if (method) {
                 start_login(method);
             }
@@ -2122,6 +2123,11 @@ void Map2Cmd(unsigned char *data, int len)
          */
         mapdata_clear_old(x, y);
 
+        if (debug_protocol)
+            LOG(LOG_INFO, "Map2Cmd", "(%d, %d) ", x, y);
+
+        bool first_label = true;
+
         /* Inner loop is for the data on the space itself */
         while (pos < len) {
             type = data[pos++];
@@ -2135,11 +2141,36 @@ void Map2Cmd(unsigned char *data, int len)
             /* Clear the space */
             if (type == MAP2_TYPE_CLEAR) {
                 mapdata_clear_space(x, y);
+                if (debug_protocol)
+                    LOG(LOG_INFO, "Map2Cmd", " clear");
                 continue;
             } else if (type == MAP2_TYPE_DARKNESS) {
                 value = data[pos++];
                 mapdata_set_darkness(x, y, value);
+                if (debug_protocol)
+                    LOG(LOG_INFO, "Map2Cmd", " darkness %d", value);
                 continue;
+            } else if (type == MAP2_TYPE_LABEL) {
+                if (space_len != 7) {
+                    // protocol error
+                    abort();
+                }
+                unsigned int lenp2 = data[pos++];
+                unsigned char subtype = data[pos++];
+                unsigned int len = data[pos++];
+                char buf[256];
+                strncpy(buf, (const char *)(data+pos), len);
+                buf[len] = '\0';
+                if (debug_protocol)
+                    LOG(LOG_INFO, "Map2Cmd", " label %d %s (%d)", subtype, buf, first_label);
+                if (first_label) {
+                    int px = pl_pos.x + x;
+                    int py = pl_pos.y + y;
+                    mapdata_clear_label(px, py);
+                }
+                mapdata_add_label(x, y, subtype, buf);
+                first_label = false;
+                pos += len;
             } else if (type >= MAP2_LAYER_START && type < MAP2_LAYER_START+MAXLAYERS) {
                 int layer, opt;
 
@@ -2160,6 +2191,8 @@ void Map2Cmd(unsigned char *data, int len)
                 pos += 2;
                 if (!(value&FACE_IS_ANIM)) {
                     mapdata_set_face_layer(x, y, value, layer);
+                    if (debug_protocol)
+                        LOG(LOG_INFO, "Map2Cmd", " layer %d face %d", layer, value);
                 }
 
                 if (space_len > 2) {
@@ -2178,7 +2211,15 @@ void Map2Cmd(unsigned char *data, int len)
                     mapdata_set_smooth(x, y, opt, layer);
                 }
                 continue;
-            } /* if image layer */
+            } else {
+                // unhandled type, we're still responsible for checking the length
+                if (space_len != 7) {
+                    pos += space_len;
+                } else {
+                    int len = data[pos++];
+                    pos += len;
+                }
+            }
         } /* while pos<len inner loop for space */
     } /* While pos<len outer loop */
     mapupdatesent = 0;
@@ -2499,7 +2540,7 @@ void AccountPlayersCmd(char *buf, int len) {
             continue;
         }
         pos++;
-        if ((pos +flen) > len || flen>=MAX_BUF) {
+        if ((pos +flen) > len) { // flen always < MAX_BUF by virtue of being 8 bits
             LOG(LOG_ERROR,"commands.c:AccountPlayerCmd", "data overran buffer");
             return;
         }
