@@ -35,7 +35,10 @@ const char *const config_names[CONFIG_NUMS] = {
     "map_width", "map_height", "foodbeep", "darkness", "port",
     "grad_color_bars", "resistances", "smoothing", "nosplash",
     "auto_apply_container", "mapscroll", "sign_popups", "message_timestamping", "auto_afk",
-    "inv_menu"
+    "inv_menu",
+    "music_vol",
+    "server_ticks",
+    "debounce",
 };
 
 gint16 want_config[CONFIG_NUMS], use_config[CONFIG_NUMS];
@@ -159,11 +162,10 @@ static void init_config() {
     want_config[CONFIG_POPUPS] = FALSE;
     want_config[CONFIG_PORT] = EPORT;
     want_config[CONFIG_RESISTS] = 0;
-    want_config[CONFIG_RESISTS] = 0;
     want_config[CONFIG_SHOWGRID] = FALSE;
     want_config[CONFIG_SHOWICON] = FALSE;
     want_config[CONFIG_SIGNPOPUP] = TRUE;
-    want_config[CONFIG_SMOOTH] = 0;
+    want_config[CONFIG_SMOOTH] = FALSE;
     want_config[CONFIG_SOUND] = TRUE;
     want_config[CONFIG_SPLASH] = TRUE;
     want_config[CONFIG_SPLITINFO] = FALSE;
@@ -173,6 +175,9 @@ static void init_config() {
     want_config[CONFIG_TRIMINFO] = FALSE;
     want_config[CONFIG_AUTO_AFK] = 300;
     want_config[CONFIG_INV_MENU] = TRUE;
+    want_config[CONFIG_MUSIC_VOL] = 100;
+    want_config[CONFIG_SERVER_TICKS] = FALSE;
+    want_config[CONFIG_DEBOUNCE] = TRUE;
 
     for (int i = 0; i < CONFIG_NUMS; i++) {
         use_config[i] = want_config[i];
