@@ -131,6 +131,8 @@ static void clear_cells(int x, int y, int len_y) {
     }
 }
 
+extern void mapdata_clear_label(int px, int py);
+
 /**
  * Get the stored map cell at the given map coordinate.
  */
@@ -209,6 +211,7 @@ void mapdata_clear(int x, int y) {
     }
 
     mapdata_cell(px, py)->state = FOG;
+    mapdata_clear_label(px, py);
 }
 
 static void mark_resmooth(int x, int y, int layer)
@@ -800,6 +803,39 @@ void mapdata_set_smooth(int x, int y, guint8 smooth, int layer)
     }
 }
 
+void mapdata_add_label(int x, int y, int subtype, const char *label) {
+    assert(0 <= x && x < MAX_VIEW);
+    assert(0 <= y && y < MAX_VIEW);
+    if (!(x < width && y < height))
+        return;
+
+    int px = pl_pos.x + x;
+    int py = pl_pos.y + y;
+    assert(0 <= px && px < the_map.width);
+    assert(0 <= py && py < the_map.height);
+
+    if (subtype == 0) {
+        mapdata_clear_label(px, py);
+        return;
+    }
+
+    struct MapLabel *new = g_malloc(sizeof(struct MapLabel));
+    new->subtype = subtype;
+    new->label = g_strdup(label);
+    new->next = mapdata_cell(px, py)->label;
+    mapdata_cell(px, py)->label = new;
+    mapdata_cell(px, py)->need_update = 1;
+}
+
+void mapdata_clear_label(int px, int py) {
+    while (mapdata_cell(px, py)->label != NULL) {
+        struct MapLabel *next = mapdata_cell(px, py)->label->next;
+        g_free(mapdata_cell(px, py)->label->label);
+        g_free(mapdata_cell(px, py)->label);
+        mapdata_cell(px, py)->label = next;
+    }
+}
+
 /**
  * Prepare a map cell, which may contain old fog of war data, for new visible
  * map data. Call this when Map2 is imminently going to provide an update for
@@ -826,6 +862,7 @@ void mapdata_clear_old(int x, int y)
             expand_clear_face_from_layer(px, py, i);
         }
         mapdata_cell(px, py)->darkness = 0;
+        mapdata_clear_label(px, py);
     }
 
     mapdata_cell(px, py)->state = VISIBLE;
@@ -1385,14 +1422,14 @@ void mapdata_animation(void)
         }
     }
 
-    for (x=0; x < CURRENT_MAX_VIEW; x++) {
-        for (y=0; y < CURRENT_MAX_VIEW; y++) {
+    for (x=0; x < MIN(width, CURRENT_MAX_VIEW); x++) {
+        for (y=0; y < MIN(height, CURRENT_MAX_VIEW); y++) {
             struct MapCell *map_space = mapdata_cell(pl_pos.x + x, pl_pos.y + y);
             
             /* Short cut some processing here.  It makes sense to me
              * not to animate stuff out of view
              */
-            if (map_space->state == VISIBLE) {
+            if (map_space->state != VISIBLE) {
                 continue;
             }
 
