@@ -20,6 +20,7 @@
 
 #include "client.h"
 
+#include <assert.h>
 #include <errno.h>
 
 #include "script.h"
@@ -210,9 +211,7 @@ static guint16 GetUShort_String(const unsigned char data[static 2]) // We want a
  * @return     Return true if we think we have a full packet, 0 if we have
  *             a partial packet, or -1 if an error occurred.
  */
-bool SockList_ReadPacket(GSocketConnection c[static 1], SockList sl[static 1],
-                         size_t len, GError** error) {
-    GInputStream* in = g_io_stream_get_input_stream(G_IO_STREAM(csocket.fd));
+bool SockList_ReadPacket(GInputStream in[static 1], SockList sl[static 1], size_t len, GError** error) {
     gsize read;
     if (!g_input_stream_read_all(in, sl->buf, 2, &read, NULL, error)) {
         return false;
@@ -257,12 +256,9 @@ int cs_print_string(GSocketConnection* fd, const char *str, ...)
 
     SockList_Init(&sl, buf);
     va_start(args, str);
-    sl.len += vsnprintf((char*)sl.buf + sl.len, MAX_BUF-sl.len, str, args);
+    sl.len += vsnprintf((char*)sl.buf + sl.len, MAX_BUF-sl.len-3, str, args); // need 2 bytes for length, 1 for null termination
     va_end(args);
-    // Adjust sl.len to account for when we overflow the buffer.
-    if (sl.len > MAX_BUF - 3) {
-        sl.len = MAX_BUF - 3;
-    }
+    assert(sl.len <= MAX_BUF - 3);
 
     script_monitor_str((char*)sl.buf);
 
