@@ -533,15 +533,27 @@ void script_kill(const char *params)
     script_dead(i);
 }
 
-#ifdef WIN32
+void script_killall_wrapper(const char *params)
+{
+	script_killall();
+}
+
 void script_killall(void)
 {
-    while (num_scripts > 0) {
-        GenerateConsoleCtrlEvent(CTRL_BREAK_EVENT, scripts[0].pid);
-        script_dead(0);
-    }
-}
+	const char message_template[] = "Tried to kill %d scripts.";
+	char message[sizeof(message_template) + 10];
+	snprintf(message, sizeof(message), message_template, num_scripts);
+	while (num_scripts > 0) {
+#ifndef WIN32
+		kill(scripts[num_scripts - 1].pid, SIGHUP);
+		script_dead(num_scripts - 1);
+#else
+		GenerateConsoleCtrlEvent(CTRL_BREAK_EVENT, scripts[0].pid);
+		script_dead(0);
 #endif /* WIN32 */
+	}
+	draw_ext_info(NDI_RED, MSG_TYPE_CLIENT, MSG_TYPE_CLIENT_SCRIPT, message);
+}
 
 void script_fdset(int *maxfd, fd_set *set)
 {
@@ -1059,9 +1071,9 @@ static void send_map(int i, int x, int y)
              " smooth %d %d %d heads %d %d %d tails %d %d %d\n",
              x, y, mapdata_cell(x, y)->darkness,
              mapdata_cell(x, y)->need_update ? 'y' : 'n',
-             mapdata_cell(x, y)->have_darkness ? 'y' : 'n',
+             mapdata_cell(x, y)->darkness != 0 ? 'y' : 'n',
              mapdata_cell(x, y)->need_resmooth ? 'y' : 'n',
-             mapdata_cell(x, y)->cleared ? 'y' : 'n',
+             mapdata_cell(x, y)->state != VISIBLE ? 'y' : 'n',
              mapdata_cell(x, y)->smooth[0], mapdata_cell(x, y)->smooth[1], mapdata_cell(x, y)->smooth[2],
              mapdata_cell(x, y)->heads[0].face, mapdata_cell(x, y)->heads[1].face, mapdata_cell(x, y)->heads[2].face,
              mapdata_cell(x, y)->tails[0].face, mapdata_cell(x, y)->tails[1].face, mapdata_cell(x, y)->tails[2].face
