@@ -1,6 +1,6 @@
 /*
  * static char *rcsid_common_script_h =
- *   "$Id: script.h 12149 2009-08-20 02:11:28Z kbulgrien $";
+ *   "$Id$";
  */
 /*
     Crossfire client, a client program for the crossfire program.
@@ -54,12 +54,11 @@ void script_init(const char *params);
 void script_list(void);
 void script_sync(int cmddiff);
 void script_kill(const char *params);
-#ifdef WIN32
+void script_killall_wrapper(const char *params);
 void script_killall(void);
-#endif
 void script_fdset(int *maxfd,fd_set *set);
 void script_process(fd_set *set);
-void script_watch(const char *cmd, const uint8 *data, const int len, const enum CmdFormat format);
+void script_watch(const char *cmd, const guint8 *data, const int len, const enum CmdFormat format);
 void script_monitor(const char *command, int repeat, int must_send);
 void script_monitor_str(const char *command);
 void script_tell(const char *params);
