@@ -298,6 +298,8 @@ struct msgctrl_data_t {
  * @} EndOf GTK V2 Message Control System.
  */
 
+extern bool arm_mapedit;
+
 /**
  * Sets attributes in the text tag from a style.  Best I can gather, there is
  * no way to take all of the attributes from a style and apply them directly to
@@ -1098,6 +1100,23 @@ void info_buffer_tick(void)
     }
 }
 
+static void launch_mapedit(char *path) {
+    char *editor = getenv("CF_MAP_EDITOR");
+    if (editor == NULL) {
+        LOG(LOG_ERROR, "mapedit", "CF_MAP_EDITOR is not defined");
+        draw_ext_info(NDI_RED, MSG_TYPE_CLIENT, MSG_TYPE_CLIENT_NOTICE, "No map editor is specified in CF_MAP_EDITOR environment variable!");
+        // If no editor, then we cannot proceed. argv[0] of null crashes the client.
+        return;
+    }
+
+    LOG(LOG_INFO, "mapedit", "Launching map editor on '%s'", path);
+    char *argv[] = {editor, path, NULL};
+    GError *err = NULL;
+    if (!g_spawn_async(NULL, argv, NULL, 0, NULL, NULL, NULL, &err)) {
+        LOG(LOG_ERROR, "mapedit", "Could not launch CF_MAP_EDITOR: %s", err->message);
+    }
+}
+
 /**
  * A callback to accept messages along with meta information color and type.
  * Unlike the GTK V1 client, we don't do anything tricky like popups with
@@ -1148,6 +1167,23 @@ message_callback(int orig_color, int type, int subtype, char *message)
      * at the end of the string in the buffer. IE. If the buffer size is 40,
      * only 39 chars can be put into it to ensure room for a null character.
      */
+    if (arm_mapedit) {
+        arm_mapedit = false;
+        char *msg_cpy = strdup(message);
+        char *ptr = msg_cpy;
+        ptr = strtok(ptr, ")");
+        if (ptr != NULL) {
+            ptr = strtok(ptr, "(");
+            ptr = strtok(NULL, "(");
+            if (ptr != NULL) {
+                launch_mapedit(ptr);
+            } else {
+                LOG(LOG_INFO, "mapedit", "Could not parse map path from '%s'", message);
+            }
+        }
+        free(msg_cpy);
+    }
+
     if ((type <  1)
             ||  (type >= MSG_TYPE_LAST)
             ||  (orig_color == NDI_UNIQUE)
@@ -1317,11 +1353,13 @@ void msgctrl_init(GtkWidget *window_root)
     GList*         list;                /* Iterator: table children         */
     guint          pane;                /* Iterator: client message panes   */
     guint          type;                /* Iterator: message types          */
-    guint          row;                 /* Attachement for current widget   */
-    gint           title_rows = -1;     /* Title rows in msgctrl_table as
-                                         * defined in layout designer.  -1
-                                         * means there are no title rows.
-                                         */
+    guint          row;                 /* Attachment for current widget    */
+    guint          title_cols;          /* Title cols in msgctrl_table      */
+    guint          title_rows;          /* Title rows in msgctrl_table      */
+                                        /* The cols/rows will describe the
+                                         * prefilled table data already in
+                                         * msgctrl_table when first loaded
+                                         * from the .ui file containing it. */
     /*
      * Get the window pointer and a pointer to the tree of widgets it contains
      */
@@ -1350,12 +1388,7 @@ void msgctrl_init(GtkWidget *window_root)
      * This assumption is unwise if client layouts begin to be implemented to
      * have fewer message panes than the code supports!
      */
-    for (list = gtk_container_get_children(GTK_CONTAINER(table)); list; list = list->next) {
-        child = list->data;
-        if ((child->widget != 0) && (child->top_attach > title_rows)) {
-            title_rows = child->top_attach;
-        }
-    }
+     gtk_table_get_size(table, &title_rows, &title_cols);
 
     /*
      * The table is defined in the dialog created with the design tool, but
