@@ -37,6 +37,8 @@ typedef int KeyCode;                    /**< Undefined type */
 #include "gtk2proto.h"
 #include "p_cmd.h"
 
+const int COUNT_MAX = 10000000;
+
 struct keybind;
 static int keybind_remove(struct keybind *entry);
 static void keybind_free(struct keybind **entry);
@@ -884,11 +886,15 @@ static void parse_key(char key, guint32 keysym) {
         present_flags |= KEYF_MOD_META;
     }
 
+    const bool numkey = key >= '0' && key <= '9';
+
     kb = keybind_find(keysym, present_flags, 0); /* char scope */
     if (kb == NULL) {
         kb = keybind_find(keysym, present_flags, 1);    /* global scope */
     }
-    if (kb != NULL) {
+    if (kb != NULL &&
+            // ignore number bindings if count is focused
+            !(numkey && gtk_widget_has_focus(GTK_WIDGET(spinbutton_count)))) {
         if (kb->flags & KEYF_EDIT) {
             strcpy(cpl.input_text, kb->command);
             cpl.input_state = Command_Mode;
@@ -903,22 +909,22 @@ static void parse_key(char key, guint32 keysym) {
             if (cpl.fire_on) {
                 snprintf(buf, sizeof(buf), "fire %s", kb->command);
                 fire_dir(kb->direction);
+                return;
             } else if (cpl.run_on) {
                 snprintf(buf, sizeof(buf), "run %s", kb->command);
                 run_dir(kb->direction);
-            } else {
-                extended_command(kb->command);
+                return;
             }
-        } else {
-            extended_command(kb->command);
         }
+        extended_command(kb->command);
+        gtk_spin_button_set_value(GTK_SPIN_BUTTON(spinbutton_count), (float) cpl.count);
         return;
     }
 
-    if (key >= '0' && key <= '9') {
+    if (numkey) {
         cpl.count = cpl.count * 10 + (key - '0');
-        if (cpl.count > 100000) {
-            cpl.count %= 100000;
+        if (cpl.count > COUNT_MAX) {
+            cpl.count %= COUNT_MAX;
         }
         gtk_spin_button_set_value(GTK_SPIN_BUTTON(spinbutton_count), (float) cpl.count);
         return;
@@ -1647,7 +1653,7 @@ void keyfunc(GtkWidget *widget, GdkEventKey *event, GtkWidget *window) {
                             debounce = false;
                         }
 
-                        if (!debounce) {
+                        if (!use_config[CONFIG_DEBOUNCE] || !debounce) {
                             parse_key(event->string[0], event->keyval);
                             debounce = true;
                         }
@@ -1820,6 +1826,7 @@ void on_entry_commands_activate(GtkEntry *entry, gpointer user_data) {
             cur_history_position %= MAX_HISTORY;
             scroll_history_position = cur_history_position;
             extended_command(entry_text);
+            gtk_spin_button_set_value(GTK_SPIN_BUTTON(spinbutton_count), (float) cpl.count);
         }
     }
     gtk_entry_set_text(GTK_ENTRY(entry), "");
