@@ -60,6 +60,8 @@ extern bool profile_latency;
 extern int MINLOG;
 bool debug_protocol;
 
+static bool playing = FALSE; // Set to true when main window is up, gates self-generated ticks
+
 static char *connect_server = NULL;
 
 static gboolean script_launch(const gchar *option_name, const gchar *value, gpointer data, GError **error);
@@ -201,11 +203,6 @@ void client_tick(guint32 tick) {
     info_buffer_tick();
     inventory_tick();
     mapdata_animation();
-    g_idle_add(redraw, NULL);
-
-    if (!is_afk && use_config[CONFIG_AUTO_AFK] != 0 && time(NULL) > (last_command_sent + use_config[CONFIG_AUTO_AFK])) {
-        auto_afk();
-    }
 }
 /**
  * Handles client shutdown.
@@ -225,8 +222,9 @@ static gboolean do_network(GObject *stream, gpointer data) {
     int pollret;
 
     if (!client_is_connected()) {
+        // It would be better to call gtk_main_quit() after client_disconnect(). But that function
+        // is in common, so we just check here.
         gtk_main_quit();
-        LOG(LOG_INFO, "main.c::do_network", "Trying to do network when not connected.");
         return FALSE;
     }
 
@@ -248,6 +246,21 @@ static gboolean do_network(GObject *stream, gpointer data) {
     return TRUE;
 }
 
+static gboolean self_tick(gpointer data) {
+    if (playing) {
+        g_idle_add(redraw, NULL);
+
+        if (!is_afk && use_config[CONFIG_AUTO_AFK] != 0 && time(NULL) > (last_command_sent + use_config[CONFIG_AUTO_AFK])) {
+            auto_afk();
+        }
+        if (!want_config[CONFIG_SERVER_TICKS]) {
+            client_tick(0);
+        }
+        return TRUE;
+    }
+    return FALSE;
+}
+
 /**
  * Set up, enter, and exit event loop. Blocks until event loop returns.
  */
@@ -256,18 +269,13 @@ static void event_loop() {
     g_timeout_add(250, G_SOURCE_FUNC(do_scriptout), NULL);
 #endif
 
+    // Set up network callback
     GSource *net_source = client_get_source();
-    if (net_source == NULL) {
-        error_dialog("Server unexpectedly disconnected",
-                     "The server unexpectedly disconnected.");
-        client_disconnect();
-        return;
-    }
+    g_assert(net_source != NULL); // crashes if input stream is not a pollable input stream
     g_source_set_callback(net_source, (GSourceFunc)do_network, NULL, NULL);
     g_source_attach(net_source, NULL);
-    gtk_main();
 
-    LOG(LOG_DEBUG, "event_loop", "Disconnected");
+    gtk_main();
 }
 
 /**
@@ -312,6 +320,7 @@ static void parse_args(int argc, char *argv[]) {
  * and information relevant to the error condition.
  */
 void error_dialog(char *error, char *message) {
+    LOG(LOG_ERROR, error, message);
     GtkWidget *dialog = gtk_message_dialog_new(
             NULL, GTK_DIALOG_DESTROY_WITH_PARENT,
             GTK_MESSAGE_ERROR, GTK_BUTTONS_CLOSE, "%s", error);
@@ -369,7 +378,6 @@ static void init_ui() {
     dialog_xml = gtk_builder_new();
     if (!gtk_builder_add_from_file(dialog_xml, DIALOG_FILENAME, &error)) {
         error_dialog("Couldn't load UI dialogs.", error->message);
-        g_warning("Couldn't load UI dialogs: %s", error->message);
         g_error_free(error);
         exit(EXIT_FAILURE);
     }
@@ -464,10 +472,19 @@ static void init_ui() {
  * new loginmethod, or after character is selected.
  */
 void show_main_client() {
+    if (!playing) {
+        // Prevent double-time if show_main_client() is called multiple times (it's possible)
+        g_timeout_add(1000/8, G_SOURCE_FUNC(self_tick), NULL);
+    }
+    playing = TRUE;
     hide_all_login_windows();
     gtk_widget_show(window_root);
     clear_stat_mapping();
     map_init(window_root);
+
+    // Reset auto-AFK data.
+    is_afk = false;
+    last_command_sent = time(NULL);
 }
 
 /**
@@ -475,6 +492,7 @@ void show_main_client() {
  * comes up (before logging in, or after having applied a bed).
  */
 void hide_main_client() {
+    playing = FALSE;
     gtk_widget_hide(window_root);
     remove_item_inventory(cpl.ob);
     /*
@@ -544,15 +562,18 @@ int main(int argc, char *argv[]) {
             cpl.input_state = Playing;
         }
 
-        client_negotiate(use_config[CONFIG_SOUND]);
-        if (serverloginmethod) {
-            account_show_login();
-        } else {
-            show_main_client();
-        }
+        if (client_negotiate(use_config[CONFIG_SOUND])) {
+            if (serverloginmethod) {
+                account_show_login();
+            } else {
+                show_main_client();
+            }
 
-        /* The event_loop will block until connection to the server is lost. */
-        event_loop();
+            /* The event_loop will block until connection to the server is lost. */
+            event_loop();
+        } else {
+            LOG(LOG_ERROR, "main", "Failed to negotiate connection with server");
+        }
 
         hide_main_client();
 
@@ -562,6 +583,10 @@ int main(int argc, char *argv[]) {
          */
         reset_image_data();
         client_reset();
+
+        if (connect_server != NULL) {
+            break;
+        }
     }
 }
 
